<?php

namespace modules\products\vps\integrations\YourName;

/**
 * -------------------------------------------------------------------------------------
 * YourName
 * 
 * Author		: 		
 * Copyright	: 		
 * Version 		:	
 * 
 * CHANGE LOG:
 * -------------------------------------------------------------------------------------
 *  yyyy-mm-yy		Author 		Initial version
 * -------------------------------------------------------------------------------------
 */

class YourName
{
    // use these to connect to VPS platform
    public $ServerURL, $ServerUser, $ServerPass;
    
    public $Error;
    public $Warning;
    public $Success;

    public function __construct()
    {        
        $this->Error    = array();
        $this->Warning  = array();
        $this->Success  = array();
        
        $this->loadLanguageArray(LANGUAGE_CODE);
    }
    
    /**
	 * Use this function to prefix all errors messages with your VPS platform
	 * 
	 * @param 	string	 $message	The error message 
	 * @return 	boolean 			Always false
	 */
    private function __parseError($message)
	{
		$this->Error[] = 'YourName: ' . $message;
		return FALSE;	
	}
    
	/**
	 * Get list of templates from the VPS platform
	 * 
	 * @return 	array 	List of templates
	 */    
    public function getTemplateList()
    {
        /**
    	 * Step 1) get template list
    	 */
        $response = TRUE;
        
        /**
    	 * Step 2) provide feedback to HostFact
    	 */
        if($response)
        {
            if(count($response) == 0)
            {
                return $this->__parseError(__('node has no templates', 'YourName'));
            }
            
            $templates  = array();
            $i          = 0;
            // loop through templates and build return array
            foreach($response->templates as $template)
            {
                $templates[$i]['templateid']    = $template->id;
                $templates[$i]['templatename']  = $template->name;
                $templates[$i]['memory']        = $template->ram;
                $templates[$i]['diskspace']     = $template->disk;
                $templates[$i]['cpucores']      = $template->vcpus;
                $i++;
            }
            
            return $templates;
        }
        
        return FALSE;
    }
    
    /**
	 * Perform an action on the VPS (eg pause, start, restart)
	 * 
     * @param 	string	 $vps_id      ID of the VPS on the VPS platform
     * @param 	string	 $action      Type of action
	 * @return 	boolean               True on success; False otherwise.
	 */
    public function doServerAction($vps_id, $action)
    {        
        switch($action)
        {
            case 'pause':
                /**
            	 * Step 1) send pause command
            	 */
                $response = TRUE;
            break;
            
            case 'start':
                /**
            	 * Step 1) send start command
            	 */
                $response = TRUE;
            break;
            
            case 'restart':
                /**
            	 * Step 1) send restart command
            	 */
                $response = TRUE;
            break;
        }
        
        /**
    	 * Step 2) provide feedback to HostFact
    	 */

        if($response)
        {
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }
    
    /**
	 * Get template details from VPS platform
	 * 
     * @param 	string   $template_id   ID of the template on the VPS platform
	 * @return 	array    Array with template details
	 */    
    public function getTemplate($template_id)
    {        
        /**
    	 * Step 1) get template
    	 */
        $response = TRUE;
        
        /**
    	 * Step 2) provide feedback to HostFact
    	 */
         
        if(!$response)
        {
            return $this->__parseError(__('node has no templates', 'YourName'));
        }
        
        $template  = array();
        $template['templateid']    = $response->template->id;
        $template['templatename']  = $response->template->name;
        $template['memory']        = $response->template->ram;
        $template['diskspace']     = $response->template->disk;
		$template['bandwidth']     = $response->template->bandwidth
        $template['cpucores']      = $response->template->vcpus;
        
        return $template;
    }
    
	/**
	 * Get list of images from the VPS platform
	 * 
	 * @return 	array 	List of images
	 */    
    public function getImageList()
    {
        /**
    	 * Step 1) get images list
    	 */
        $response = TRUE;
        
        /**
    	 * Step 2) provide feedback to HostFact
    	 */                
        if($response)
        {
            $images = array();
            $i      = 0;
            // loop through images and build return array
            foreach($response->images as $image)
            {                
                $images[$i]['imageid']    = $image->id;
                $images[$i]['imagename']  = $image->name;
                $i++;
            }
            
            if(count($images) == 0)
            {
                return $this->__parseError(__('node has no images', 'YourName'));
            }
            
            return $images;
        }
        
        return FALSE;
    }
    
    /**
	 * Validate the VPS server login credentials
	 * 
	 * @return 	boolean               True on success; False otherwise.
	 */    
    public function validateLogin()
    {
        // available variables:
        $this->ServerURL;
        $this->ServerUser;
        $this->ServerPass;
        
        /**
    	 * Step 1) send login or get command to check if login credentials are correct
         * 
    	 */
         
         $response = TRUE;
         
         return $response;
    }
    
    /**
     * This function makes it possible to provide additional settings or notes on the create and edit page of a VPS server within HostFact.
     * This may be necessary if more information is needed than just the URL, username and password of the platform. 
	 * 
     * @param   string     $edit_or_show   edit|show; determines if we are adding/editing or showing a VPS server 
	 * @return 	string     $html           input HTML
	 */
    public function showSettingsHTML($edit_or_show = 'edit')
	{
        $html = '';
        /**
    	 * Step 1) build html based on $edit_or_show
         * you can use $this->ServerSettings->InputName to request the settings variabeles, see examples below
         * 
    	 */
        // example:
//		if($edit_or_show == 'show')
//		{
//			// Show page
//			$html = '<strong class="title2">Project ID (Tenant ID)</strong>' .  
//					'<span class="title2_value">' . 
//					((isset($this->ServerSettings->ProjectID)) ? htmlspecialchars($this->ServerSettings->ProjectID) : '') . 
//					'</span>' ;
//		}
//		else
//		{
//			// add/edit page
//            $html = '<strong class="title">Project ID (Tenant ID)</strong>' . 
//					'<input type="text" name="module[vps][Settings][ProjectID]" class="text1 size1" value="' . htmlspecialchars($this->ServerSettings->ProjectID) . '" />';
//		}
        
        /**
    	 * Step 2) provide feedback to HostFact
    	 */ 
        
		return $html;
	}

    /**
	 * Create a VPS on the VPS platform
	 *  
	 * @return 	array               Return array with VPS ID on success; False on fail;
	 */    
    public function createVPS()
    {
        // available variables:
        $this->VPSName;
        $this->Password;
        $this->MemoryMB;
        $this->CPUCores;
        $this->DiskSpaceGB;
        $this->TemplateID;
        $this->Image;
        
        /**
    	 * Step 1) send create command
         * 
    	 */
        $response = TRUE;         

        /**
    	 * Step 2) provide feedback to HostFact
    	 */ 
        if($response && isset($response->server->id) && $response->server->id)
        {
            $vps = array();
            $vps['id'] = $response->server->id;
                                                 
            return $vps;
        }
        else
        {
            return FALSE;
        }
    }
    
    /**
	 * Remove a VPS from the VPS platform
	 * 
     * @param 	string	 $vps_id      ID of the VPS on the VPS platform
	 * @return 	boolean               True on success; False otherwise.
	 */
    public function delete($vps_id)
    {
        /**
    	 * Step 1) send delete command
         * 
    	 */
        $response = TRUE;         

        /**
    	 * Step 2) provide feedback to HostFact
    	 */
        if($response)
        {
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }
    
    /**
	 * Function to support multiple languages for return messages
     * use __('your message', 'YourName'); to translate a message based on the language of HostFact
	 * 
     * @param 	string	 $language_code   Language code
	 */
   	public function loadLanguageArray($language_code)
	{
		$_LANG = array();

		switch($language_code)
		{
			case 'nl_NL':
                $_LANG['node gave no response']                 = 'Server gaf geen antwoord terug.';
                $_LANG['node returned error']                   = 'Server gaf een error terug.';
                $_LANG['node returned wrong data']              = 'Server gaf een antwoord terug, maar niet de benodigde data.';
                $_LANG['node has no images']                    = 'Server heeft geen images';
                $_LANG['node has no templates']                 = 'Server heeft geen templates';
			break;
            
			default: // In case of other language, use English
                $_LANG['node gave no response']                 = 'No response from node';
                $_LANG['node returned error']                   = 'Node returned an error.';
                $_LANG['node returned wrong data']              = 'Node returned incorrect data';
                $_LANG['node has no images']                    = 'Node has no images';
                $_LANG['node has no templates']                 = 'Node has no templates';
			break;
		}
		
		// Save to global array
		global $_module_language_array;
		$_module_language_array['YourName'] = $_LANG;
	}
    
    /**
	 * Suspend or unsuspend a VPS on the VPS platform
	 * 
     * @param 	string	   $vps_id         ID of the VPS on the VPS platform
     * @param   string     $action         suspend|unsuspend
	 * @return 	boolean                    True on success; False otherwise.
	 */    
    public function suspend($vps_id, $action)
    {
        if(!$this->__login())
        {
            return FALSE;
        }
        
        switch($action)
        {            
            case 'suspend':
                /**
                 * Step 1) send suspend command
                 * 
                 */
                 $response = TRUE;
            break;
            
            case 'unsuspend':
                /**
                 * Step 1) send unsuspend command
                 * 
                 */
                 $response = TRUE;
            break;
        }
        
        /**
    	 * Step 2) provide feedback to HostFact
    	 */
        if($response)
        {
            return TRUE;
        }
        else
        {
            return FALSE; 
        }
    }

    /**
	 * Get details of a VPS by ID
	 * 
     * @param 	string	   $vps_id         ID of the VPS on the VPS platform
	 * @return 	array                      Array with VPS information
	 */     
    public function getVPSDetails($vps_id)
    {
        /**
         * Step 1) send get VPS command
         * 
         */
        $response = TRUE;
        
        /**
    	 * Step 2) provide feedback to HostFact
    	 */
        if($response)
        {
            $vps_details = array();
            $vps_details['status']      = $this->__convertStatus($response->server->status);
            $vps_details['hostname']    = $response->server->name;
            $vps_details['ipaddress']   = $response->server->address; // not required
            
            return $vps_details;
        }

        return FALSE;
    }
    
    /**
	 * Get details of a VPS by hostname
	 * 
     * @param 	string	   $hostname       Hostname of the VPS on the VPS platform
	 * @return 	array      $vps            Array with VPS information
	 */     
    public function getVPSByHostname($hostname)
    {
        /**
         * Step 1) send get server command, if there ain't a specific command to get server by hostname, retrieve all servers and loop through them
         * 
         */
        $response = TRUE;

        /**
    	 * Step 2) provide feedback to HostFact
    	 */
        $vps = array();
        if($response && is_array($response->servers) && count($response->servers) > 0)
        {
            // loop through servers
            foreach($response->servers as $server)
            {
                if($hostname == $server->name)
                {
                    $vps['id']           = $server->id;
                    $vps['status']       = $this->__convertStatus($server->status);
                }
            }
        }
        
        if(isset($vps['id']))
        {
            return $vps;
        }
        
        return FALSE;
    }
    
    /**
	 * When a VPS is created from HostFact, this function is regularly called to check it's status
	 * 
     * @param 	string	   $vps_id         ID of the VPS on the VPS platform
	 * @return 	string                     active|building|error; Return status
	 */     
    public function doPending($vps_id)
    {
        /**
         * Step 1) send get VPS command (we need the VPS status)
         * 
         */
        $response = TRUE;
        
        /**
    	 * Step 2) provide feedback to HostFact
         *         based on the VPS status you get from the response, return either active, building or error         
    	 */
        if($response)
        {
            switch($this->__convertStatus($response->server->status))
            {
                case 'active':
                    return 'active';
                break;
                
                case 'building':
                    return 'building';
                break;
                
                default: 
                    return 'error';
                break;
            }
        }
        
        return 'error';
    }
    
    /**
	 * Use this function to convert VPS statusses from the VPS platform to a status HostFact can handle
	 * 
	 * @param 	string	 $status	Status returned from a VPS platform command of a VPS
	 * @return 	string 			    Converted status
	 */    
    private function __convertStatus($status)
    {
        switch($status)
        {
            // VPS is active
            case '':
                $new_status = 'active';                
            break;
            
            // VPS is rebooting
            case '':
                $new_status = 'reboot';                
            break;
            
            // VPS is paused
            case '':
                $new_status = 'paused';                
            break;
            
            // VPS has been created and is building
            case '':
                $new_status = 'building';                
            break;
            
            // VPS error
            case '':
                $new_status = 'error';                
            break;
            
            default:
                $new_status = '';
            break;
        }       
        
        return $new_status;
    }
}
